// Note: The order here must match the ordinal declarations in lzhamdll.def!
LZHAM_DLL_FUNC_NAME(lzham_get_version)
LZHAM_DLL_FUNC_NAME(lzham_set_memory_callbacks)
LZHAM_DLL_FUNC_NAME(lzham_compress_init)
LZHAM_DLL_FUNC_NAME(lzham_compress_reinit)
LZHAM_DLL_FUNC_NAME(lzham_compress)
LZHAM_DLL_FUNC_NAME(lzham_compress2)
LZHAM_DLL_FUNC_NAME(lzham_compress_deinit)
LZHAM_DLL_FUNC_NAME(lzham_compress_memory)
LZHAM_DLL_FUNC_NAME(lzham_decompress_init)
LZHAM_DLL_FUNC_NAME(lzham_decompress)
LZHAM_DLL_FUNC_NAME(lzham_decompress_deinit)
LZHAM_DLL_FUNC_NAME(lzham_decompress_memory)
LZHAM_DLL_FUNC_NAME(lzham_decompress_reinit)
LZHAM_DLL_FUNC_NAME(lzham_z_version)
LZHAM_DLL_FUNC_NAME(lzham_z_deflateInit)
LZHAM_DLL_FUNC_NAME(lzham_z_deflateInit2)
LZHAM_DLL_FUNC_NAME(lzham_z_deflateReset)
LZHAM_DLL_FUNC_NAME(lzham_z_deflate)
LZHAM_DLL_FUNC_NAME(lzham_z_deflateEnd)
LZHAM_DLL_FUNC_NAME(lzham_z_deflateBound)
LZHAM_DLL_FUNC_NAME(lzham_z_compress)
LZHAM_DLL_FUNC_NAME(lzham_z_compress2)
LZHAM_DLL_FUNC_NAME(lzham_z_compressBound)
LZHAM_DLL_FUNC_NAME(lzham_z_inflateInit)
LZHAM_DLL_FUNC_NAME(lzham_z_inflateInit2)
LZHAM_DLL_FUNC_NAME(lzham_z_inflateReset)
LZHAM_DLL_FUNC_NAME(lzham_z_inflate)
LZHAM_DLL_FUNC_NAME(lzham_z_inflateEnd)
LZHAM_DLL_FUNC_NAME(lzham_z_uncompress)
LZHAM_DLL_FUNC_NAME(lzham_z_error)
