
files_c=[
 'C/7zCrc.c',
 'C/7zCrcOpt.c',
 'C/Alloc.c',
 'C/Bra86.c',
 'C/CpuArch.c',
 'C/LzFind.c',
 'C/LzFindMt.c',
 'C/LzmaDec.c',
 'C/LzmaEnc.c',
 'C/Lzma86Dec.c',
 'C/Lzma86Enc.c',
 'C/Threads.c',
]

files_cpp=[
 'CPP/7zip/Bundles/LzmaCon/LzmaAlone.cpp',
 'CPP/7zip/Common/CWrappers.cpp',
 'CPP/7zip/Common/CreateCoder.cpp',
 'CPP/7zip/Common/FileStreams.cpp',
 'CPP/7zip/Common/FilterCoder.cpp',
 'CPP/7zip/Common/MethodProps.cpp',
 'CPP/7zip/Common/OutBuffer.cpp',
 'CPP/7zip/Common/StreamUtils.cpp',
 'CPP/7zip/Compress/LzmaDecoder.cpp',
 'CPP/7zip/Compress/LzmaEncoder.cpp',
 'CPP/7zip/Compress/LzmaRegister.cpp',
 'CPP/7zip/UI/Common/Bench.cpp',
 'CPP/7zip/UI/Console/BenchCon.cpp',
 'CPP/7zip/UI/Console/ConsoleClose.cpp',
 'CPP/Common/CRC.cpp',
 'CPP/Common/CommandLineParser.cpp',
 'CPP/Common/CrcReg.cpp',
 'CPP/Common/C_FileIO.cpp',
 'CPP/Common/IntToString.cpp',
 'CPP/Common/MyString.cpp',
 'CPP/Common/MyVector.cpp',
 'CPP/Common/MyWindows.cpp',
 'CPP/Common/NewHandler.cpp',
 'CPP/Common/StringConvert.cpp',
 'CPP/Common/StringToInt.cpp',
 'CPP/Windows/PropVariant.cpp',
 'CPP/Windows/System.cpp',
 'CPP/myWindows/wine_date_and_time.cpp',
]

# 'CPP/Windows/FileIO.cpp',

