
files_c=[
 'C/7zCrc.c',
 'C/7zCrcOpt.c',
 'C/Alloc.c',
 'C/CpuArch.c',
 'C/Ppmd7.c',
 'C/Ppmd7Dec.c',
]

files_cpp=[
 'CPP/7zip/Common/InBuffer.cpp',
 'CPP/7zip/Common/OutBuffer.cpp',
 'CPP/7zip/Common/StreamUtils.cpp',
 'CPP/7zip/Compress/CodecExports.cpp',
 'CPP/7zip/Compress/DllExportsCompress.cpp',
 'CPP/7zip/Compress/LzOutWindow.cpp',
 'CPP/7zip/Compress/Rar1Decoder.cpp',
 'CPP/7zip/Compress/Rar2Decoder.cpp',
 'CPP/7zip/Compress/Rar3Decoder.cpp',
 'CPP/7zip/Compress/Rar3Vm.cpp',
 'CPP/7zip/Compress/Rar5Decoder.cpp',
 'CPP/7zip/Compress/RarCodecsRegister.cpp',
 'CPP/Common/CRC.cpp',
 'CPP/Common/MyVector.cpp',
 'CPP/Common/MyWindows.cpp',
]

